@@ -7,7 +7,7 @@ use diagnostics;
 require Exporter;
 our @ISA = qw(Exporter);
 our @EXPORT = qw( auction );
-our $VERSION = '0.35';
+our $VERSION = '0.40';
 
 #Variables global to the package	
 my $maximize_total_benefit;
@@ -27,7 +27,7 @@ sub auction { #							=> default values
 				maximize_total_benefit  => 0,         # 0: minimize_total_benefit ; 1: maximize_total_benefit
 				inicial_stepsize        => undef,     # auction algorithm terminates with a feasible assignment if the problem data are integer and stepsize < 1/min(N,M)				
 				inicial_price           => 0,
-				verbose                 => 3,         # level of verbosity, 0: quiet; 1, 2, 3, 4, 9, 10: debug information.				
+				verbose                 => 3,         # level of verbosity, 0: quiet; 1, 2, 3, 4, 5, 8, 10: debug information.
 				@_,                                   # argument pair list goes here
 				);
        
@@ -91,7 +91,6 @@ sub auction { #							=> default values
    
 		%assignned_object = ();
 		%assignned_person = ();
-		%objects_desired_by_this = ();
 		$iter_count_local = 0;
 
 		while ( (scalar keys %assignned_person) < $max_size ){ # while there is at least one element not assigned.
@@ -107,7 +106,7 @@ sub auction { #							=> default values
 				foreach my $per ( sort { $a <=> $b } keys %assignned_person){ push @assignment, $assignned_person{$per}; }
 				foreach my $obj ( sort { $a <=> $b } keys %price_object    ){ push @prices, $price_object{$obj}; }
 				my $assig_count = scalar @assignment;
-				print "\n *** \$iter_count_global = $iter_count_global ; \$assig_count = $assig_count ; \$epsilon = $epsilon ; \@assignment = (@assignment) ; \@prices = (@prices) \n\n";
+				print " *** \$iter_count_global = $iter_count_global ; \$assig_count = $assig_count ; \$epsilon = $epsilon ; \@assignment = (@assignment) ; \@prices = (@prices) \n\n";
 			
 				for my $i ( -1 .. $#matrix ) {
 					if ($i >= 0){ printf " %2s  [", $i; } else{ printf "object"; }
@@ -154,14 +153,14 @@ sub auction { #							=> default values
 
 	for my $i ( 0 .. $original_max_size - 1 ) {
 	for my $j ( 0 .. $original_max_size - 1 ) {
-      next if ($seeN{$i} or $seeM{$j});  
-      $output_index[$i] = $j;
-	  $seeN{$i}++;
-	  $seeM{$j}++;
-	  last;
+		next if ($seeN{$i} or $seeM{$j});  
+		$output_index[$i] = $j;
+		$seeN{$i}++;
+		$seeM{$j}++;
+		last;
 	}}   
    
-	if ( $args{verbose} >= 10 ){
+	if ( $args{verbose} >= 8 ){
 		printf "\$optimal_benefit = $optimal_benefit ; \$iter_count_global = $iter_count_global ; \$epsilon = %.4g ; \@output_index = (@output_index) ; \@assignment = (@assignment) ; \@prices = (@prices) \n", $epsilon; 
 	}   
 	print_screen_messages( \@matrix, \@matrix_index, \@matrix_input, \@output_index, $optimal_benefit, $args{verbose}, $epsilon ) ;
@@ -206,7 +205,7 @@ sub delete_multiple_columns { # if the column elements do not change the final r
 	  }
    }
 
-   if ( $verbose >= 7 ){
+   if ( $verbose >= 5 ){
       print "\n";
       for my $i ( 0 .. $#{$matrix_ref} ) {
          print " [";
@@ -224,33 +223,33 @@ sub delete_multiple_columns { # if the column elements do not change the final r
       for my $j ( 0 .. $#{$matrix_ref->[$i]} ) {
 	     undef ( $matrix_ref->[$i]->[$j] ) if ( defined $intersection_columns{$j} and $intersection_columns{$j} >= $min_size );
 		 if ( $i == 0 and defined $intersection_columns{$j} and $intersection_columns{$j} >= $min_size ){
-		    printf " N = $min_size ; M = $max_size ; j = %2s ; \$intersection_columns{$j} = $intersection_columns{$j} \n", $j if ( $verbose >= 7 );
+		    printf " N = $min_size ; M = $max_size ; j = %2s ; \$intersection_columns{$j} = $intersection_columns{$j} \n", $j if ( $verbose >= 5 );
 		 }
 		 if ( $i == 0 and ( not defined $intersection_columns{$j} or $intersection_columns{$j} < $min_size ) ){
-		    $index_correlation{$idx} = $j;
-		    printf " N = $min_size ; M = $max_size ; j = %2s ; \$index_correlation{$idx} = $index_correlation{$idx} \n", $j if ( $verbose >= 7 );
+			$index_correlation{$idx} = $j;
+			printf " N = $min_size ; M = $max_size ; j = %2s ; \$index_correlation{$idx} = $index_correlation{$idx} \n", $j if ( $verbose >= 5 );
 			$idx++;
 		 }
       }
    }
    
-   for my $i ( 0 .. $#{$matrix_ref} ) {
-      @{$matrix_ref->[$i]} = grep { defined($_) } @{$matrix_ref->[$i]};
-   }
+	for my $i ( 0 .. $#{$matrix_ref} ) {
+		@{$matrix_ref->[$i]} = grep { defined($_) } @{$matrix_ref->[$i]};
+	}
   
-   if ( $verbose >= 7 ){
-      print "\n";
-      for my $i ( 0 .. $#{$matrix_ref} ) {
-         print " [";
-         for my $j ( 0 .. $#{$matrix_ref->[$i]} ) {
-            printf(" %2.0f  ", $matrix_ref->[$i]->[$j] );
-         }
-         print "]\n";
-      }
-	  print "\n";
-   }
+	if ( $verbose >= 5 ){
+		print "\n";
+		for my $i ( 0 .. $#{$matrix_ref} ) {
+			print " [";
+			for my $j ( 0 .. $#{$matrix_ref->[$i]} ) {
+				printf(" %2.0f  ", $matrix_ref->[$i]->[$j] );
+			}
+			print "]\n";
+		}
+		print "\n";
+	}
    
-   $max_size = $max_size - $number_of_columns_deleted;
+	$max_size = $max_size - $number_of_columns_deleted;
 }
 
 sub print_screen_messages {
@@ -303,7 +302,7 @@ sub print_screen_messages {
    
       my $index_length = length($original_max_size);   
 
-	  if ( $verbose >= 4 ){
+	  if ( $verbose >= 3 ){
       printf " modified matrix %d x %d:\n", $#matrix + 1, $#{$matrix[0]} + 1;	  
       for my $i ( 0 .. $#matrix ) {
          print " [";
@@ -377,7 +376,7 @@ sub get_matrix_info {
    my $range = $max_matrix_value - $min_matrix_value;         # $range >= 0
       $range = 1 if ($range == 0);
    
-   if ( $verbose >= 6 ){
+   if ( $verbose >= 4 ){
       print "\n min_matrix_value = $min_matrix_value ; max_matrix_value = $max_matrix_value ; range = $range ; matrix_spaces = $matrix_spaces ; decimals = $decimals \n";
    }
    
@@ -409,14 +408,14 @@ sub auctionRound {
 	my @matrix = @$matrix_ref;
 	my %info; my %seen_bid;
 
-	if ( $verbose >= 10 ){
+	if ( $verbose >= 8 ){
 		print " Start: Matrix Size N x M: $min_size x $max_size ; Num Iterations: $iter_count_global ; iter_count_local = $iter_count_local ; epsilon: $epsilon \n";
 		foreach my $person ( sort { $a <=> $b } keys %assignned_person ){
 			my $object = $assignned_person{$person};
-			printf " \$assignned_person{%2s} --> object %2s --> \$price_object{%2s} = $price_object{$object} \n", $person, $object, $object;
+			printf " \$assignned_person{%3s} --> object %3s --> \$price_object{%3s} = $price_object{$object} \n", $person, $object, $object;
 		}
 		foreach my $object ( sort { $a <=> $b } keys %price_object ){
-			printf " \$price_object{%2s} = $price_object{$object} \n", $object;
+			printf " \$price_object{%3s} = $price_object{$object} \n", $object;
 		}
 		print "\n";
 	}
@@ -425,7 +424,7 @@ sub auctionRound {
 
 	for my $person ( 0 .. $max_size - 1 )
 	{		
-		last if $seen_ghost; # don't need to fill the matrix with zeros, that is, don't need to convert rectangular N x M to square matrix by padding zeroes. Need just one more row: N+1 x M
+		last if $seen_ghost; # don't need to fill the matrix with zeros, that is, don't need to convert rectangular N x M to square matrix by padding zeroes. Need just one more row: N+1 x M	
 		
 		if ( not defined $assignned_person{$person} )
 		{		
@@ -436,19 +435,21 @@ sub auctionRound {
 			##  |  ...
 			##  i  person (N - 1)    price_i_0	
 			
-			my ( $Opt01ObjForPersonI, $Opt02ObjForPersonI, $Opt03ObjForPersonI, $Opt04ObjForPersonI, $Opt05ObjForPersonI, $Opt06ObjForPersonI );
-			my ( $Opt01ValForPersonI, $Opt02ValForPersonI, $Opt03ValForPersonI, $Opt04ValForPersonI, $Opt05ValForPersonI, $Opt06ValForPersonI );
+			my ( $Opt01ObjForPersonI, $Opt02ObjForPersonI, $Opt03ObjForPersonI );
+			my ( $Opt01ValForPersonI, $Opt02ValForPersonI, $Opt03ValForPersonI ) = ( -10 * exp ($max_matrix_value), -10 * exp ($max_matrix_value), -10 * exp ($max_matrix_value) );
 			my ( $bid01ForPersonI, $bid02ForPersonI );
             
 			# "The set of objects to which each person i has been assigned in the last few assignments does not change much as the algorithm proceeds."
 			# Each person is assigned to just a relatively small number of objects during the execution of the auctionRound.
 			# My implementation is different from Libor Bus and Pavel Tvrdik: 'Towards auction algorithms for large dense assignment problems' (see reference 2).
-			# My working sets @objects_with_greater_benefits are updated dynamically by %objects_desired_by_this.						
+			# My working sets @objects_with_greater_benefits are updated dynamically by $objects_desired_by_this{$person} considering the price update of objects.					
 			
-			my @objects_with_greater_benefits = keys %{$objects_desired_by_this{$person}}; # sort { $a <=> $b } keys %{$objects_desired_by_this{$person}}	            			
-			my @not_assignned_objects = grep { not defined $assignned_object{$_} } @objects_with_greater_benefits;
+			my @objects_with_greater_benefits = keys %{$objects_desired_by_this{$person}}; # sort { $a <=> $b }			
+			my @updated_price = grep { $objects_desired_by_this{$person}{$_} == $price_object{$_} } @objects_with_greater_benefits;
+			
+			# There is at least one object in the @objects_with_greater_benefits list whose price is updated ? use old list : generate new list;
 
-			for my $object ( @not_assignned_objects > 0 ? @objects_with_greater_benefits : ( 0 .. $max_size - 1 ) )		
+			for my $object ( @updated_price >= 1 ? @objects_with_greater_benefits : ( 0 .. $max_size - 1 ) )		
 			#for my $object ( 0 .. $max_size - 1 )			
 			{				
 				$seen_ghost++ if ( not defined $matrix[$person] and $min_size < $max_size );
@@ -457,14 +458,8 @@ sub auctionRound {
 				
 				my $curVal = $matrix_value - $price_object{$object};
 				
-				if ( (not defined $Opt01ValForPersonI) || ($curVal > $Opt01ValForPersonI) )
+				if ( $curVal > $Opt01ValForPersonI )
 				{
-					$Opt06ValForPersonI = $Opt05ValForPersonI;
-					$Opt06ObjForPersonI = $Opt05ObjForPersonI;
-					$Opt05ValForPersonI = $Opt04ValForPersonI;
-					$Opt05ObjForPersonI = $Opt04ObjForPersonI;
-					$Opt04ValForPersonI = $Opt03ValForPersonI;
-					$Opt04ObjForPersonI = $Opt03ObjForPersonI;
 					$Opt03ValForPersonI = $Opt02ValForPersonI;
 					$Opt03ObjForPersonI = $Opt02ObjForPersonI;
 					$Opt02ValForPersonI = $Opt01ValForPersonI;
@@ -472,54 +467,21 @@ sub auctionRound {
 					$Opt01ValForPersonI = $curVal;
 					$Opt01ObjForPersonI = $object;
 				}
-				elsif ( (not defined $Opt02ValForPersonI) || ($curVal > $Opt02ValForPersonI) )
+				elsif ( $curVal > $Opt02ValForPersonI )
 				{
-					$Opt06ValForPersonI = $Opt05ValForPersonI;
-					$Opt06ObjForPersonI = $Opt05ObjForPersonI;
-					$Opt05ValForPersonI = $Opt04ValForPersonI;
-					$Opt05ObjForPersonI = $Opt04ObjForPersonI;
-					$Opt04ValForPersonI = $Opt03ValForPersonI;
-					$Opt04ObjForPersonI = $Opt03ObjForPersonI;
 					$Opt03ValForPersonI = $Opt02ValForPersonI;
 					$Opt03ObjForPersonI = $Opt02ObjForPersonI;
 					$Opt02ValForPersonI = $curVal;
 					$Opt02ObjForPersonI = $object;
 				}
-				elsif ( (not defined $Opt03ValForPersonI) || ($curVal > $Opt03ValForPersonI) )
+				elsif ( $curVal > $Opt03ValForPersonI )
 				{
-					$Opt06ValForPersonI = $Opt05ValForPersonI;
-					$Opt06ObjForPersonI = $Opt05ObjForPersonI;
-					$Opt05ValForPersonI = $Opt04ValForPersonI;
-					$Opt05ObjForPersonI = $Opt04ObjForPersonI;
-					$Opt04ValForPersonI = $Opt03ValForPersonI;
-					$Opt04ObjForPersonI = $Opt03ObjForPersonI;
 					$Opt03ValForPersonI = $curVal;
 					$Opt03ObjForPersonI = $object;
 				}
-				elsif ( (not defined $Opt04ValForPersonI) || ($curVal > $Opt04ValForPersonI) )
-				{
-					$Opt06ValForPersonI = $Opt05ValForPersonI;
-					$Opt06ObjForPersonI = $Opt05ObjForPersonI;
-					$Opt05ValForPersonI = $Opt04ValForPersonI;
-					$Opt05ObjForPersonI = $Opt04ObjForPersonI;
-					$Opt04ValForPersonI = $curVal;
-					$Opt04ObjForPersonI = $object;
-				}
-				elsif ( (not defined $Opt05ValForPersonI) || ($curVal > $Opt05ValForPersonI) )
-				{
-					$Opt06ValForPersonI = $Opt05ValForPersonI;
-					$Opt06ObjForPersonI = $Opt05ObjForPersonI;
-					$Opt05ValForPersonI = $curVal;
-					$Opt05ObjForPersonI = $object;
-				}
-				elsif ( (not defined $Opt06ValForPersonI) || ($curVal > $Opt06ValForPersonI) )
-				{
-					$Opt06ValForPersonI = $curVal;
-					$Opt06ObjForPersonI = $object;
-				}
-				
-				if ( $verbose >= 10 ){
-					printf " personI = %2s ; objectJ = %2s ; \$curVal %10.5f = \$matrix[%2s][%2s] %3.0f - \$price_object{%2s} %10.5f \n", $person, $object, $curVal, $person, $object, $matrix_value, $object, $price_object{$object};
+
+				if ( $verbose >= 8 ){
+					printf " personI = %3s ; objectJ = %3s ; \$curVal %12.5f = \$matrix[%3s][%3s] %3.0f - \$price_object{%3s} %10.5f \n", $person, $object, $curVal, $person, $object, $matrix_value, $object, $price_object{$object};
 				}				
 			}
 			
@@ -533,26 +495,32 @@ sub auctionRound {
 				$bid02ForPersonI = $Opt02ValForPersonI - $Opt03ValForPersonI + $epsilon; # calculate $bid02ForPersonI to try to predict the values of the next round
 				$bid02ForPersonI = 2 * $epsilon if ( $Opt01ValForPersonI > ($Opt02ValForPersonI + $epsilon) or $Opt02ValForPersonI > ($Opt03ValForPersonI + $epsilon) );
 			
-				$info{$Opt02ObjForPersonI}{$bid02ForPersonI} = $person if ( ++$seen_bid{$Opt01ObjForPersonI}{$bid01ForPersonI} >= 2 );
+				$info{$Opt02ObjForPersonI}{$bid02ForPersonI} = $person if ( ++$seen_bid{$Opt01ObjForPersonI}{$bid01ForPersonI} >= 2 );							
 				
-				$objects_desired_by_this{$person}{$Opt01ObjForPersonI} = 1; # information about the most desired objects
-				$objects_desired_by_this{$person}{$Opt02ObjForPersonI} = 1 if (defined $Opt02ObjForPersonI);
-				$objects_desired_by_this{$person}{$Opt03ObjForPersonI} = 1 if (defined $Opt03ObjForPersonI);
-				$objects_desired_by_this{$person}{$Opt04ObjForPersonI} = 1 if (defined $Opt04ObjForPersonI);
-				$objects_desired_by_this{$person}{$Opt05ObjForPersonI} = 1 if (defined $Opt05ObjForPersonI);
-				$objects_desired_by_this{$person}{$Opt06ObjForPersonI} = 1 if (defined $Opt06ObjForPersonI);								
+				if ( @updated_price == 0 ){ # all prices are outdated
+				
+					$objects_desired_by_this{$person}{$_} = $price_object{$_} for ( @objects_with_greater_benefits ); # update prices for all objects in the list
+
+					delete $objects_desired_by_this{$person} if ( @objects_with_greater_benefits > 15 ) ; # delete old list
+
+					$objects_desired_by_this{$person}{$Opt01ObjForPersonI} = $price_object{$Opt01ObjForPersonI}; # information about the most desired objects
+					$objects_desired_by_this{$person}{$Opt02ObjForPersonI} = $price_object{$Opt02ObjForPersonI} if (defined $Opt02ObjForPersonI);
+					$objects_desired_by_this{$person}{$Opt03ObjForPersonI} = $price_object{$Opt03ObjForPersonI} if (defined $Opt03ObjForPersonI);
+				}
 			}
 			
-			if ( $verbose >= 10 ){
-				my @array = sort { $a <=> $b } keys %{$objects_desired_by_this{$person}};
+			if ( $verbose >= 8 ){
+				my @old_list = sort { $a <=> $b } @objects_with_greater_benefits;
+				my @new_list = sort { $a <=> $b } keys %{$objects_desired_by_this{$person}};
+				my @best_3_objects = ( $Opt01ObjForPersonI, $Opt02ObjForPersonI, $Opt03ObjForPersonI );
 
-				printf "<> PersonI = %2s ; \@objects_with_greater_benefits = (@objects_with_greater_benefits) ; object eligible to bid --> \@not_assignned_objects = (@not_assignned_objects) : %2s > 0 ? \n", $person, scalar @not_assignned_objects;
-				printf "<> PersonI = %2s ; objects desired by this person = (@array) \n", $person;
-				printf "<> PersonI = %2s chose ObjectJ = %2s ; \$bid01ForPersonI %10.5f = \$Opt01ValForPersonI %.5f - \$Opt02ValForPersonI %.5f + \$epsilon %.5f \n", $person, $Opt01ObjForPersonI, $bid01ForPersonI, $Opt01ValForPersonI, $Opt02ValForPersonI, $epsilon;
-				printf "<> PersonI = %2s chose ObjectJ = %2s ; \$bid02ForPersonI %10.5f = \$Opt02ValForPersonI %.5f - \$Opt03ValForPersonI %.5f + \$epsilon %.5f \n", $person, $Opt02ObjForPersonI, $bid02ForPersonI, $Opt02ValForPersonI, $Opt03ValForPersonI, $epsilon if ( defined $bid02ForPersonI );
+				printf "<> PersonI = %3s ; objects desired by this person (old list) = (@old_list) ; objects whose prices were not updated = (@updated_price) : %2s >= 1 ? \n", $person, scalar @updated_price;
+				printf "<> PersonI = %3s ; objects desired by this person (new list) = (@new_list) ; \@best_3_objects = (@best_3_objects) \n", $person if ( defined $Opt03ObjForPersonI );
+				printf "<> PersonI = %3s chose ObjectJ = %3s ; \$bid01ForPersonI %10.5f = \$Opt01ValForPersonI %.5f - \$Opt02ValForPersonI %.5f + \$epsilon %.5f \n", $person, $Opt01ObjForPersonI, $bid01ForPersonI, $Opt01ValForPersonI, $Opt02ValForPersonI, $epsilon;
+				printf "<> PersonI = %3s chose ObjectJ = %3s ; \$bid02ForPersonI %10.5f = \$Opt02ValForPersonI %.5f - \$Opt03ValForPersonI %.5f + \$epsilon %.5f \n", $person, $Opt02ObjForPersonI, $bid02ForPersonI, $Opt02ValForPersonI, $Opt03ValForPersonI, $epsilon if ( defined $bid02ForPersonI );
 			}
 		}
-	}		
+	}
 	
 	foreach my $object ( keys %info ){                                 # for each object, choose only the first bid.
 	foreach my $bid    ( sort { $b <=> $a } keys %{$info{$object}} ){  # descending order!!! --> the first bid is the highest bid.
@@ -562,7 +530,7 @@ sub auctionRound {
 	   	   	   
 		if ( defined $other_person ) {
 		  
-			if ( $verbose >= 10 ){
+			if ( $verbose >= 8 ){
 				print " ***--> PersonI $person was assigned objectJ $object. Before that, remove the objectJ $object from personI $other_person  --> delete \$assignned_person{$other_person} \n";
 			}
 		  
@@ -577,24 +545,24 @@ sub auctionRound {
 	   
 		$assignned_person{$person} = $object;
 		$assignned_object{$object} = $person;
-	   
+
 		$price_object{$object} += $bid;
 	   
-		if ( $verbose >= 10 ){
+		if ( $verbose >= 8 ){
 			printf " --> Assigning to personI = %2s the objectJ = %2s with highestBidForJ = %10.5f and update the price vector ; \$assignned_person{%2s} = %2s ; \$price_object{%2s} = %.5f \n", $person, $object, $bid, $person, $assignned_person{$person}, $object, $price_object{$object};	
 		}
 	   
 		last;  # next object, choose only the highest bid for each object
 	}}
 	
-	if ( $verbose >= 10 ){
+	if ( $verbose >= 8 ){
 		print "\n Final: Matrix Size N x M: $min_size x $max_size ; Num Iterations: $iter_count_global ; iter_count_local = $iter_count_local ; epsilon: $epsilon \n";
 		foreach my $person ( sort { $a <=> $b } keys %assignned_person ){
 			my $object = $assignned_person{$person};
-			printf " \$assignned_person{%2s} --> object %2s --> \$price_object{%2s} = $price_object{$object} \n", $person, $object, $object;
+			printf " \$assignned_person{%3s} --> object %3s --> \$price_object{%3s} = $price_object{$object} \n", $person, $object, $object;
 		}
 		foreach my $object ( sort { $a <=> $b } keys %price_object ){
-			printf " \$price_object{%2s} = $price_object{$object} \n", $object;
+			printf " \$price_object{%3s} = $price_object{$object} \n", $object;
 		}
 		print "\n";
 	}
@@ -790,7 +758,7 @@ __END__
  maximize_total_benefit => 0,    0: minimize the total benefit ; 1: maximize the total benefit.
  inicial_stepsize       => 1,    auction algorithm terminates with a feasible assignment if the problem data are integer and stepsize < 1/min(N,M).
  inicial_price          => 0,			
- verbose                => 3,    print messages on the screen, level of verbosity, 0: quiet; 1, 2, 3, 4, 9, 10: debug information.
+ verbose                => 3,    print messages on the screen, level of verbosity, 0: quiet; 1, 2, 3, 4, 5, 8, 10: debug information.
 
 =head1 EXPORT
 
@@ -829,7 +797,7 @@ __END__
 =head1 AUTHOR
 
     Claudio Fernandes de Souza Rodrigues
-	March 31, 2018
+	April 03, 2018
 	Sao Paulo, Brasil
 	claudiofsr@yahoo.com
 	
